import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/field/date/formatter.dart';

/// Mixin supports 2 digits day or month, 2 digits month or day and 4 digits year text fields.
mixin DateFieldStateMixin {
  final TextEditingController _controller = TextEditingController();
  final FocusNode _focusNode = FocusNode();
  late final bool _initialValid;
  late final int _maxLength;
  int? year, month, day;

  /// Call this on building the UI.
  Widget buildMixin(final BuildContext context,
          {final TextInputAction action = TextInputAction.next,
          final bool clearable = false,
          final Widget clearIcon = const Icon(Icons.clear),
          required final BoxDecoration decoration,
          required final BoxDecoration errorDecoration,
          final String? hint,
          required final TextStyle style,
          required final TextStyle errorStyle}) =>
      container(
          decoration: valid ? decoration : errorDecoration,
          textField: TextField(
              controller: _controller,
              decoration: InputDecoration(
                  border: InputBorder.none,
                  counterText: '',
                  hintText: hint,
                  suffixIcon: clearable
                      ? IconButton(
                          onPressed: () => _controller.clear(), icon: clearIcon)
                      : null),
              focusNode: _focusNode,
              inputFormatters: [
                FilteringTextInputFormatter.digitsOnly,
                formatter
              ],
              keyboardType: Platform.isAndroid
                  ? TextInputType.datetime
                  : TextInputType.number,
              maxLength: _maxLength,
              style: _initialValid && valid ? style : errorStyle,
              textInputAction: action));

  Widget container(
      {required final BoxDecoration decoration,
      required final TextField textField});

  /// Listener added to the controller.
  void _controllerListener() {
    onChanged();
    if (_controller.text.length == _maxLength) {
      _validate();
    }
  }

  /// Call this on diposing.
  void disposeMixin() {
    _controller.removeListener(_controllerListener);
    _controller.dispose();
    _focusNode.removeListener(_focusNodeListener);
    _focusNode.dispose();
  }

  /// Listener added to the focus node.
  void _focusNodeListener() {
    if (!_focusNode.hasFocus) {
      if (_controller.text.length < _maxLength) {
        valid = _controller.text.isEmpty;
        refresh();
      } else {
        _validate();
      }
    }
  }

  /// Call this on initializing.
  void initStateMixin(
      {required final String? initialDate,
      required final bool initialValid,
      required final String separator}) {
    _initialValid = initialValid;
    valid = initialValid;

    /// 8 digits and 2 separators between.
    _maxLength = 8 + separator.length * 2;

    _controller.addListener(_controllerListener);
    _focusNode.addListener(_focusNodeListener);

    if (initialDate != null) {
      _controller.text = initialDate;
      _controller.selection =
          TextSelection.collapsed(offset: initialDate.length);
    }
  }

  /// Call this on changing text. _validate() should be outside of this implementation.
  void onChanged();

  /// Call this on refreshing the UI.
  void refresh();

  /// Split text to obtaining day, month and year.
  List<String> split({required final String separator}) =>
      _controller.text.split(separator);

  /// Validate the day, month and year.
  void _validate() {
    if (year == null || month == null || day == null) {
      valid = false;
    } else {
      final DateTime date = DateTime(year!, month!, day!);
      valid = date.year == year && date.month == month && date.day == day;
    }
    refresh();
  }

  DateFieldFormatter get formatter;

  set valid(final bool valid);

  bool get valid;
}
