import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_custom_form_component/widget/field/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/money/formatter.dart';
import 'package:flutter_custom_form_component/widget/field/one_value/mixin.dart';
import 'package:flutter_custom_form_component/widget/field/one_value/widget.dart';

/// Text field supports money format.
class MoneyFieldWidget extends OneValueFieldWidget {
  late final String _decimalSeparator, _thousandSeparator;
  late final String? _symbol;

  MoneyFieldWidget(
      {final TextInputAction action = TextInputAction.next,
      final BoxDecoration? decoration,
      final bool enabled = true,
      final BoxDecoration? errorDecoration,
      final TextStyle? errorStyle,
      final String? hint,
      final double? initialValue,
      final Key? key,
      final Function(dynamic value)? onChanged,
      final String? symbol,
      final TextStyle? style,
      final bool usingPeriodInThousand = true})
      : super(
            action: action,
            decoration: decoration,
            enabled: enabled,
            errorDecoration: errorDecoration,
            errorStyle: errorStyle,
            hint: hint,
            initialValue: initialValue,
            key: key,
            onChanged: onChanged,
            style: style) {
    _symbol = symbol;

    /// Using '.' for separating thousands and ',' for separating decimal. Or the opposite.
    _decimalSeparator = usingPeriodInThousand ? ',' : '.';
    _thousandSeparator = usingPeriodInThousand ? '.' : ',';
  }

  @override
  State<StatefulWidget> createState() => _State();
}

class _State extends State<MoneyFieldWidget>
    with FieldMixin, OneValueFieldMixin {
  late final MoneyFormatter _formatter;

  @override
  Widget build(final BuildContext context) => container(
      decoration: valid ? widget.decoration : widget.errorDecoration,
      textField: TextField(
          controller: controller,
          decoration:
              InputDecoration(border: InputBorder.none, hintText: widget.hint),
          enabled: widget.enabled,
          inputFormatters: [
            FilteringTextInputFormatter.allow(
                RegExp('[0-9${widget._decimalSeparator}]')),
            _formatter
          ],
          keyboardType: Platform.isAndroid
              ? const TextInputType.numberWithOptions(decimal: true)
              : TextInputType.text,
          style: valid ? widget.style : widget.errorStyle,
          textInputAction: widget.action));

  @override
  void dispose() {
    disposeMixin();
    super.dispose();
  }

  @override
  void initState() {
    super.initState();
    initStateMixin(
        initialValue: widget.initialValue, onChanged: widget.onChanged);
    _formatter = MoneyFormatter(
        decimalSeparator: widget._decimalSeparator,
        symbol: widget._symbol,
        thousandSeparator: widget._thousandSeparator);
  }

  @override
  String format(final dynamic value) {
    if (value == null) {
      return '';
    }

    /// Replace common decimal separator with widget._decimalSeparator
    String string =
        value!.toString().replaceFirst('.', widget._decimalSeparator);
    final List<String> strings = string.split(widget._decimalSeparator);
    if (int.tryParse(strings[1]) == 0) {
      string = strings[0];
    }
    return _formatter.format(string);
  }

  @override
  double get value {
    String string = controller.text
        .replaceAll(widget._thousandSeparator, '')
        .replaceFirst(widget._decimalSeparator, '.');
    if (widget._symbol != null) {
      string = string.replaceFirst(widget._symbol!, '');
    }
    return double.tryParse(string) ?? 0.0;
  }
}
